@@ -45,6 +45,11 @@ Alejandro Dubrovsky <s328940@student.uq.edu.au>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
+#ifdef LIBZIP_SUPPORTED
+#include "zip.h"
+#endif
+
+
 #include "tea_text_document.h" // Needs to be above for t_note_page struct
 #include "rox_strings.h"
 #include "tea_defs.h"
@@ -53,11 +58,6 @@ Alejandro Dubrovsky <s328940@student.uq.edu.au>
 #include "tea_enc.h"
 #include "interface.h"
 
-/* To avoid implicit declaration warnings */
-void run_unitaz (t_note_page *page, gint sort_type, gboolean case_insensetive);
-void run_extract_words (t_note_page *page);
-//void doc_spaces_to_tabs (t_note_page *doc, gint tabsize);
-//void doc_tabs_to_spaces (t_note_page *doc, gint tabsize);
 
 //*  Glimmer - misc.c
 /*
@@ -183,10 +183,10 @@ size_t tNumber2Roman (guint uiNumber, gint bUpperCase, gchar *szOutput)
              }
 
        if (uiNumber == 0)
-         {
-          *outp = '\0';
-          return (size_t)(outp - szOutput);
-         }
+          {
+           *outp = '\0';
+           return (size_t)(outp - szOutput);
+          }
 
        q = p + 1;
        uiNextVal = uiValue / (uint)(gchar)*q;
@@ -212,7 +212,7 @@ size_t tNumber2Roman (guint uiNumber, gint bUpperCase, gchar *szOutput)
 gchar* get_file_ver_name (const gchar *name)
 {
   if (! name)
-     return;
+     return NULL;
 
   if (! g_file_test (name, G_FILE_TEST_EXISTS))
      return NULL;
@@ -254,35 +254,6 @@ GList* add_to_glist_combined_str (GList *list, const gchar *key, const gchar *va
 }
 
 
-static gint sort_node (const gchar *a, const gchar *b)
-{
-  if (! a && ! b)
-     return 0;
-  else if (! a)
-      return -1;
-  else if (! b)
-      return 1;
-
-  gchar *t1 = g_utf8_collate_key (a, -1);
-  gchar *t2 = g_utf8_collate_key (b, -1);
-  gint result = strcmp (t1, t2);
-
-  g_free (t1);
-  g_free (t2);
-
-  return result;
-}
-
-
-GList* glist_strings_sort (GList *list)
-{
-  if (! list)
-     return NULL;
-
-  return g_list_sort (list, (GCompareFunc) sort_node);
-}
-
-
 GList* glist_from_string_sep (const gchar *string, const gchar *sep)
 {
   if (! string || ! sep)
@@ -377,7 +348,7 @@ GList* load_file_to_glist (const gchar *filename)
      return NULL;
 
   gsize length;
-  gchar *buf;
+  gchar *buf = NULL;
 
   if (! g_file_get_contents (filename, &buf, &length, NULL))
      return NULL;
@@ -390,17 +361,15 @@ GList* load_file_to_glist (const gchar *filename)
 
 void glist_save_to_file (GList *list, const gchar *filename)
 {
-  if (! filename || ! list)
+  if (! filename)
      return;
 
-  /*
-  if (g_list_length (list) == 0)
+  if ((! list) ||  (g_list_length (list) == 0))
      {
-      dbm ("unlink");
       unlink (filename);
       return;
      }
-*/
+
   FILE *out = fopen (filename, "w+");
   if (! out)
       return;
@@ -694,13 +663,25 @@ gchar* str_file_read (const gchar *filename)
 }
 
 
-typedef struct {
-               gchar *word;
-               gint count;
-              } t_struct_word;
+static gint sort_node (const gchar *a, const gchar *b)
+{
+  if (! a && ! b)
+     return 0;
+  else if (! a)
+      return -1;
+  else if (! b)
+      return 1;
 
+  gchar *t1 = g_utf8_collate_key (a, -1);
+  gchar *t2 = g_utf8_collate_key (b, -1);
+  gint result = strcmp (t1, t2);
+
+  g_free (t1);
+  g_free (t2);
+
+  return result;
+}
 
-static GList *l_words;
 
 gint sort_node_m1 (t_struct_word *a, t_struct_word *b)
 {
@@ -727,6 +708,32 @@ gint sort_node_m2 (t_struct_word *a, t_struct_word *b)
 }
 
 
+gint sort_node_m3 (const gchar *sa, const gchar *sb)
+{
+  if (! sa && ! sb)
+     return 0;
+  else if (! sa)
+      return -1;
+  else if (! sb)
+      return 1;
+
+  gchar *a = g_utf8_casefold (sa, -1);
+  gchar *b = g_utf8_casefold (sb, -1);
+
+  gchar *t1 = g_utf8_collate_key (a, -1);
+  gchar *t2 = g_utf8_collate_key (b, -1);
+
+  gint result = strcmp (t1, t2);
+
+  g_free (t1);
+  g_free (t2);
+  g_free (a);
+  g_free (b);
+
+  return result;
+}
+
+
 gint sort_node_m4 (const gchar *a, const gchar *b)
 {
   glong x = g_utf8_strlen (a, - 1);
@@ -743,6 +750,24 @@ gint sort_node_m4 (const gchar *a, const gchar *b)
 }
 
 
+GList* glist_strings_sort (GList *list)
+{
+  if (! list)
+     return NULL;
+
+  return g_list_sort (list, (GCompareFunc) sort_node);
+}
+
+
+GList* sort_list_case_insensetive (GList *list)
+{
+  if (! list)
+     return NULL;
+
+  return g_list_sort (list, (GCompareFunc) sort_node_m3);
+}
+
+
 GList* glist_word_sort_mode (GList *list, gint mode)
 {
   if (mode == 0) return g_list_sort (list, (GCompareFunc) sort_node_m1);
@@ -760,184 +785,6 @@ void free_word_data (gpointer data)
 }
 
 
-void str_walk_extract_word (gpointer key,
-                            gpointer value,
-                            gpointer user_data)
-{
-  l_words = g_list_prepend (l_words, key);
-}
-
-
-void run_extract_words (t_note_page *page)
-{
-  if (! page)
-     return;
-
-  GList *list = NULL;
-  GtkTextIter a;
-  GtkTextIter b;
-
-  l_words = NULL;
-  GHashTable *words = g_hash_table_new (g_str_hash, g_str_equal);
-
-  gchar *s;
-  gchar *t;
-  GtkTextIter start;
-  GtkTextIter end;
-
-  if (doc_has_selection (page))
-     gtk_text_buffer_get_selection_bounds (page->text_buffer, &start, &end);
-  else
-      gtk_text_buffer_get_bounds (page->text_buffer, &start, &end);
-
-  do
-    {
-     if (gtk_text_iter_starts_word (&start))
-        a = start;
-
-     if (gtk_text_iter_ends_word (&start))
-        {
-         s = gtk_text_buffer_get_text (page->text_buffer, &a, &start, FALSE);
-
-         t = g_hash_table_lookup (words, s);
-         if (! t)
-            g_hash_table_insert (words, s, s);
-         else
-             g_free (s);
-         }
-     }
-  while (gtk_text_iter_forward_char (&start));
-
-  if (words)
-     g_hash_table_foreach (words, str_walk_extract_word, NULL);
-
-  GList *tmplist = g_list_first (g_list_reverse (l_words));
-  while (tmplist)
-        {
-         list = g_list_prepend (list, tmplist->data);
-         tmplist = g_list_next (tmplist);
-        }
-
-  gchar *sr = string_from_glist (g_list_reverse (list));
-  cur_text_doc = doc_ins_to_new (sr);
-
-  g_free (sr);
-  glist_strings_free (list);
-  g_list_free (l_words);
-  g_hash_table_destroy (words);
-}
-
-
-void walk_by_words (gpointer key,
-                    gpointer value,
-                    gpointer user_data)
-{
-  l_words = g_list_prepend (l_words, value);
-}
-
-
-void run_unitaz (t_note_page *page, gint sort_type, gboolean case_insensetive)
-{
-  if (! page)
-     return;
-
-  GList *list = NULL;
-  GtkTextIter a;
-  GtkTextIter b;
-
-  int w_total = 0;
-  int w_unique = 0;
-  l_words = NULL;
-
-  GHashTable *words = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, free_word_data);
-
-  t_struct_word *ts = NULL;
-  t_struct_word *t = NULL;
-  gchar *s;
-  gchar *z;
-
-  GtkTextIter start;
-  GtkTextIter end;
-
-  if (doc_has_selection (page))
-     gtk_text_buffer_get_selection_bounds (page->text_buffer, &start, &end);
-  else
-      gtk_text_buffer_get_selection_bounds (page->text_buffer, &start, &end);
-
-
-  do
-    {
-     while (gtk_events_pending ())
-       gtk_main_iteration ();
-
-     if (gtk_text_iter_starts_word (&start))
-        a = start;
-
-     if (gtk_text_iter_ends_word (&start))
-        {
-         w_total++;
-         if (case_insensetive)
-            {
-             z = gtk_text_buffer_get_text (page->text_buffer, &a, &start, FALSE);
-             s = g_utf8_strdown (z, -1);
-             g_free (z);
-            }
-         else
-             s = gtk_text_buffer_get_text (page->text_buffer, &a, &start, FALSE);
-
-         t = g_hash_table_lookup (words, s);
-         if (! t)
-              {
-               ts = (t_struct_word *) g_malloc (sizeof (t_struct_word));
-               ts->count = 1;
-               ts->word = s;
-               g_hash_table_insert (words, s, ts);
-               w_unique++;
-              }
-            else
-                {
-                 t->count++;
-                 g_free (s);
-                }
-           }
-    }
-  while ( gtk_text_iter_forward_char (&start));
-
-  gint i;
-  gchar *st = NULL;
-
-  if (words)
-     g_hash_table_foreach (words, walk_by_words, NULL);
-
-  GList *tl;
-
-  list = g_list_prepend (list, g_strdup_printf (_("Text analysis results for %s:"), page->file_name_utf8));
-  list = g_list_prepend (list, g_strdup_printf (_("Words total: %d"), w_total));
-  list = g_list_prepend (list, g_strdup_printf (_("Words unique: %d"), w_unique));
-  list = g_list_prepend (list, g_strdup_printf (_("Words total / Words unique = %.6f"), (float)w_total/w_unique));
-  list = g_list_prepend (list, g_strdup_printf ("%-10s %s", _("Number:"), _("Word:")));
-
-  l_words = glist_word_sort_mode (l_words, sort_type);
-
-  GList *tmplist = g_list_first (l_words);
-  while (tmplist)
-        {
-         t = (t_struct_word *) tmplist->data;
-         st = g_strdup_printf ("%-10d %s", t->count, t->word);
-         list = g_list_prepend (list, st);
-         tmplist = g_list_next (tmplist);
-        }
-
-  gchar *sr = string_from_glist (g_list_reverse (list));
-  cur_text_doc = doc_ins_to_new (sr);
-
-  g_free (sr);
-  glist_strings_free (list);
-  g_list_free (l_words);
-  g_hash_table_destroy (words);
-}
-
-
 GList* glist_shuffle (GList *list)
 {
   if (! list)
@@ -987,7 +834,6 @@ gchar* parse_rtf_hex_str (const gchar *s)
   gint code;
   gchar *hex;
 
-
   for (i = 0; st[i]; i++)
       {
        if ((st[i] == '\\') && (st[i+1]) == '\'')
@@ -1036,47 +882,14 @@ gchar* text_load_rtf (const gchar *filename)
     return NULL;
 
   gchar *t = str_file_read (filename);
+  if (! t)
+     return NULL;
   gchar *s = parse_rtf_hex_str (t);
   g_free (t);
   return s;
 }
 
 
-gint sort_node_m3 (const gchar *sa, const gchar *sb)
-{
-  if (! sa && ! sb)
-     return 0;
-  else if (! sa)
-      return -1;
-  else if (! sb)
-      return 1;
-
-  gchar *a = g_utf8_casefold (sa, -1);
-  gchar *b = g_utf8_casefold (sb, -1);
-
-  gchar *t1 = g_utf8_collate_key (a, -1);
-  gchar *t2 = g_utf8_collate_key (b, -1);
-
-  gint result = strcmp (t1, t2);
-
-  g_free (t1);
-  g_free (t2);
-  g_free (a);
-  g_free (b);
-
-  return result;
-}
-
-
-GList* sort_list_case_insensetive (GList *list)
-{
-  if (! list)
-     return NULL;
-
-  return g_list_sort (list, (GCompareFunc) sort_node_m3);
-}
-
-
 GList* arr_to_glist (const gchar **a)
 {
   GList *l = NULL;
@@ -1172,10 +985,13 @@ modified by roxton
   if (! str)
      return;
 
-  gchar *space_pos, *ptr;
+  gchar *space_pos;
+  gchar *ptr;
   gint te = 0;
 
-  gint ptr_offset, line_begin_offset, space_pos_offset;
+  gint ptr_offset;
+  gint line_begin_offset;
+  gint space_pos_offset;
 
   line_begin_offset = ptr_offset = space_pos_offset = 0;
   space_pos = ptr = str;
@@ -1204,7 +1020,7 @@ modified by roxton
              line_begin_offset = space_pos_offset + 1;
              te = 0;
             }
-        ptr = g_utf8_next_char(ptr);
+        ptr = g_utf8_next_char (ptr);
         ptr_offset++;
        }
 }
@@ -1294,34 +1110,6 @@ gchar* rep_all_s (const gchar *s, const gchar *r)
 }
 
 
-gchar* str_replace_all (const gchar *s, const gchar *what, const gchar *to)
-{
-  if (! s)
-     return NULL;
-
-  if (! what)
-     return g_strdup (s);
-
-  if (! to)
-     return g_strdup (s);
-
-  if (! strstr (s, what))
-     return g_strdup (s);
-
-  gchar *t = g_str_replace (s, what, to);
-
-  gchar *z;
-  while (strstr (t, what))
-        {
-         z = g_str_replace (t, what, to);
-         g_free (t);
-         t = z;
-        }
-
-  return t;
-}
-
-
 //CHANGEME: change %s macro to @s?
 GList* glist_repl (GList* list, const gchar *format)
 {
@@ -1757,7 +1545,20 @@ GList* ht_to_glist (GHashTable *hash_table, gint option)
       if ((option == opt_ht_to_glist_keys) && hash_table)
          g_hash_table_foreach (hash_table, ht_to_glist_keys_cb, NULL);
 
-  return temp_glist;
+  return glist_copy_with_data (temp_glist, -1);
+}
+
+
+void save_ht_to_file (GHashTable *hash_table, const gchar *filename)
+{
+  temp_glist = NULL;
+
+  GList *l = ht_to_glist (hash_table, opt_ht_to_glist_full);
+  if (! l)
+     return;
+
+  glist_save_to_file (l, filename);
+  glist_strings_free (l);
 }
 
 
@@ -1812,6 +1613,13 @@ gchar* get_c_source_fname (const gchar *f)
 
   gchar *t = NULL;
 
+  if (g_str_has_suffix (f, ".hh"))
+     {
+      t = g_str_replace (f, ".hh", ".cc");
+      if (g_file_test (t, G_FILE_TEST_EXISTS))
+          return t;
+     }
+  else
   if (g_str_has_suffix (f, ".h"))
      {
       t = g_str_replace (f, ".h", ".cpp");
@@ -1841,6 +1649,9 @@ gchar* get_c_header_fname (const gchar *f)
   else
       if (g_str_has_suffix (f, ".c"))
          t = g_str_replace (f, ".c", ".h");
+  else
+      if (g_str_has_suffix (f, ".cc"))
+         t = g_str_replace (f, ".cc", ".hh");
 
   if (g_file_test (t, G_FILE_TEST_EXISTS))
      return t;
@@ -1912,6 +1723,13 @@ gchar* text_load_gzip (const gchar *filename)
   if (! filename)
      return NULL;
 
+#ifdef LIBZIP_SUPPORTED
+
+  if (is_ext (filename, ".zip", NULL))
+      return read_single_file_from_zip (filename);
+
+#endif
+
   gchar *cm = NULL;
 
   if (strstr (filename, ".tar."))
@@ -2035,7 +1853,6 @@ gchar* change_file_ext (const gchar *filename, const gchar *new_ext)
      return g_strconcat (filename, ".", new_ext, NULL);
 
   GList *l = glist_from_string_sep (filename, ".");
-
   GList *t = g_list_last (l);
 
   if (t && t->data)
@@ -2048,23 +1865,6 @@ gchar* change_file_ext (const gchar *filename, const gchar *new_ext)
 }
 
 
-gchar* get_file_ext (const gchar *filename)
-{
-  if (! filename)
-     return NULL;
-
-  if (! strstr (filename, "."))
-     return NULL;
-
-  GList *l = glist_from_string_sep (filename, ".");
-  GList *t = g_list_last (l);
-  gchar *f = g_strdup (t->data);
-  glist_strings_free (l);
-
-  return f;
-}
-
-
 gboolean str_in (const gchar *s, gchar *ext1, ...)
 {
   if (! s)
@@ -2171,6 +1971,18 @@ gchar* read_xml_text (const gchar *filename, const gchar *xml_file, const gchar
   if (! filename || ! xml_file || ! para)
      return NULL;
 
+#ifdef LIBZIP_SUPPORTED
+
+   gchar *xml = read_file_from_zip (filename, xml_file);
+   if (! xml)
+      return NULL;
+
+  gchar *r = xml_to_plain_text (xml, para);
+  g_free (xml);
+  return r;
+
+#else
+
   if (g_file_test (confile.tmp_dir, G_FILE_TEST_EXISTS))
      delete_dir (confile.tmp_dir);
 
@@ -2190,12 +2002,14 @@ gchar* read_xml_text (const gchar *filename, const gchar *xml_file, const gchar
   g_free (f);
   g_free (t);
   return r;
+
+#endif
 }
 
 
-gchar* read_abw_text (const gchar *filename, const gchar *xml_file, const gchar *para)
+gchar* read_abw_text (const gchar *filename, const gchar *para)
 {
-  if (! filename || ! xml_file || ! para)
+  if (! filename || ! para)
      return NULL;
 
   gchar *t = str_file_read (filename);
@@ -2320,12 +2134,18 @@ gchar* str_to_html (const gchar *buf, gboolean xhtml)
      return NULL;
 
   GList *l = glist_from_string (buf);
+  gchar *dtd;
+
+  if (! xhtml)
+     dtd = g_strdup ("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/1999/REC-html401-19991224/loose.dtd\">\n");
+  else
+      dtd = g_strdup ("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n");
 
   l = glist_repl (l, "<p class=\"p1\">%s</p>");
 
   l = g_list_prepend (l,
                       g_strconcat (
-                      "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/1999/REC-html401-19991224/loose.dtd\">\n",
+                          dtd ,
                           "<html>\n",
                           "<head>\n",
                           "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n",
@@ -2354,32 +2174,12 @@ gchar* str_to_html (const gchar *buf, gboolean xhtml)
   else
       x = str_replace_all (r, "<p class=\"p1\"></p>", "<br>");
 
+  g_free (dtd);
   g_free (r);
   return x;
 }
 
 
-gint glist_get_str_pos (GList *list, const gchar *st)
-{
-  if (! st || ! list)
-      return 0;
-
-  gint c = 0;
-
-  GList *p = g_list_first (list);
-
-  while (p)
-        {
-         if (g_utf8_collate (st, p->data) == 0)
-            break;
-         p = g_list_next (p);
-         ++c;
-        }
-
-  return c;
-}
-
-
 gchar* copied_from_browser_to_ftext (const gchar *s, const gchar *pattern)
 {
   if (! s || ! pattern)
@@ -2498,8 +2298,8 @@ gchar* tea_convert_charset (const gchar *text, const gchar *from, const gchar *t
   gsize bytes_written;
   GError *error = NULL;
 
-  if (g_utf8_collate (from, CURR_LOCALE) == 0)
-     return (g_locale_to_utf8 (text, strlen (text), &bytes_read, &bytes_written, &error));
+  if (g_utf8_collate (to, from) == 0)
+     return g_strdup (text);
 
   return (g_convert (text, strlen (text), to, from, &bytes_read, &bytes_written, &error));
 }
@@ -2839,7 +2639,7 @@ gchar* strinfile_office (const gchar *filename, const gchar *text_to_find)
      buf = read_xml_text (filename, "content.xml", "text:p");
   else
       if (check_ext (filename, ".abw"))
-         buf = read_abw_text (filename, filename, "p");
+         buf = read_abw_text (filename, "p");
       else
           if (check_ext (filename, ".kwd"))
              buf = read_xml_text (filename, "maindoc.xml", "TEXT");
@@ -2859,72 +2659,270 @@ gchar* strinfile_office (const gchar *filename, const gchar *text_to_find)
 }
 
 
-gchar* search_with_charset (const gchar *buf, const gchar *charset, const gchar *text_to_find)
+GList* glist_copy_with_data (GList *source, gint count)
 {
-  gchar *converted_text = tea_convert_charset (text_to_find, "UTF-8", text_to_find);;
-  gchar *pos = strstr (buf, converted_text);
-  g_free (converted_text);
-  return pos;
+  if (! source)
+      return NULL;
+
+  if (count == -1)
+     count = g_list_length (source);
+  else
+       count = count - 1;
+
+  GList *result = NULL;
+  GList *t = g_list_first (source);
+  gint c = 0;
+
+  while (t && (c++ < count))
+        {
+         if (t->data)
+            result = g_list_prepend (result, g_strdup (t->data));
+         t = g_list_next (t);
+        }
+
+  return g_list_reverse (result);
 }
 
 
-gboolean strinfile_bool (const gchar *filename, const gchar *text_to_find)
+GList* glist_copy_with_filter (GList *source, FilterStrFunc func)
+{
+  if (! source)
+      return NULL;
+
+  GList *result = NULL;
+  GList *t = g_list_first (source);
+  gint c = 0;
+
+  while (t)
+        {
+         if (t->data && func (t->data))
+            result = g_list_prepend (result, g_strdup (t->data));
+         t = g_list_next (t);
+        }
+
+  return g_list_reverse (result);
+}
+
+
+gchar *apply_repl_table (const gchar *text, const gchar *table_filename)
+{
+  if (! text || ! table_filename)
+     return NULL;
+
+  gchar *st = g_strdup (text);
+  GList *l = load_file_to_glist (table_filename);
+  GList *t = g_list_first (l);
+  while (t)
+        {
+         gchar *key = NULL;
+         gchar *val = NULL;
+         gchar **a = NULL;
+         a = g_strsplit (t->data, "=", 2);
+
+         if (a)
+            {
+             if (get_gchar_array_size (a) > 1)
+                {
+                 key = a[0];
+                 val = a[1];
+                 gchar *x = str_replace_all (st, key, val);
+                 if (x)
+                   {
+                    g_free (st);
+                    st = x;
+                   }
+                }
+
+             g_strfreev (a);
+            }
+
+         t = g_list_next (t);
+        }
+
+  gslist_strings_free (l);
+  return st;
+}
+
+
+gchar* str_replace_all (const gchar *input, const gchar *replace, const gchar *with)
+{
+  if (! input)
+     return NULL;
+
+  if (! replace)
+     return g_strdup (input);
+
+  if (! with)
+     return g_strdup (input);
+
+  gint len = strlen (replace);
+
+  gchar *text = g_strdup (input);
+  gchar *start = strstr (text, replace);
+
+  if (! start)
+     return g_strdup (input);
+
+  gchar *end = start + len;
+  *start = '\0';
+
+  gchar *value = g_strconcat (text, with, end, NULL);
+  g_free (text);
+
+  while (strstr (end, replace))
+        {
+         text = value;
+         start = strstr (text, replace);
+         if (! start)
+            break;
+
+        end = start + len;
+        *start = '\0';
+        gchar *x = g_strconcat (text, with, end, NULL);
+        g_free (value);
+        value = x;
+        }
+
+  return (value);
+}
+
+
+gchar* strinfile_charset (const gchar *filename, const gchar *text_to_find, const gchar *charset)
 {
   gchar *buf = str_file_read (filename);
   if (! buf)
-     return FALSE;
+     return NULL;
 
   gchar *converted_text;
   gchar *pos = NULL;
+  gchar *result = NULL;
 
-  GList *l = g_list_first (confile.iconv_encs);
-  while (l)
-        {
-         converted_text = tea_convert_charset (text_to_find, "UTF-8", l->data);
-
-        if (converted_text)
-           {
-            pos = strstr (buf, converted_text);
-            if (pos)
-              {
-               g_free (converted_text);
-               g_free (buf);
-               return TRUE;
-              }
-           }
+  converted_text = tea_convert_charset (text_to_find, "UTF-8", charset);
 
+  if (converted_text)
+      {
+       pos = strstr (buf, converted_text);
+       if (pos)
+           result = recent_item_compose (filename /* UTF-8!*/, charset, 0);
         g_free (converted_text);
-        l = g_list_next (l);
+       }
+
+  g_free (buf);
+  return result;
+}
+
+
+GList* filter_list_pattern (GList *list, const gchar *pattern, gboolean incld)
+{
+  if (! pattern || ! list)
+     return list;
+
+  GPatternSpec *pat = g_pattern_spec_new (pattern);
+
+  GList *t = NULL;
+  GList *p = g_list_first (list);
+
+  while (p)
+        {
+         if (incld)
+            {
+             if (g_pattern_match_string (pat, p->data))
+               t = g_list_prepend (t, g_strdup (p->data));
+            }
+         else
+             if (! g_pattern_match_string (pat, p->data))
+               t = g_list_prepend (t, g_strdup (p->data));
+
+         p = g_list_next (p);
         }
 
-  g_free (buf);
-  return FALSE;
+  g_pattern_spec_free (pat);
+
+  return g_list_reverse (t);
 }
 
 
-gboolean strinfile_office_bool (const gchar *filename, const gchar *text_to_find)
+gchar* read_file_from_zip (const gchar *archive_name, const gchar *file_name)
 {
   gchar *buf = NULL;
-  if (check_ext (filename, ".sxw") || check_ext (filename, ".odt"))
-     buf = read_xml_text (filename, "content.xml", "text:p");
-  else
-      if (check_ext (filename, ".abw"))
-         buf = read_abw_text (filename, filename, "p");
-      else
-          if (check_ext (filename, ".kwd"))
-             buf = read_xml_text (filename, "maindoc.xml", "TEXT");
 
-  if (! buf)
-     return NULL;
+#ifdef LIBZIP_SUPPORTED
 
-  gchar *pos = NULL;
+  int errorp;
+  struct zip *archive = zip_open (archive_name, ZIP_CHECKCONS, &errorp);
+  if (! archive)
+     {
+      dbm ("! archive");
+      return NULL;
+     }
 
-  pos = strstr (buf, text_to_find);
-  if (pos)
-      pos = recent_item_compose (filename, "UTF-8", 0);
+  struct zip_file *file_zip = zip_fopen (archive, file_name, 0);
+  if (! file_zip)
+     {
+      dbm ("! file_zip");
+      zip_close (archive);
+      return NULL;
+     }
 
-  g_free (buf);
+   struct zip_stat sb;
+   if (zip_stat (archive, file_name, 0, &sb) != 0)
+      {
+       dbm ("! zip_stat");
+       zip_fclose (file_zip);
+       zip_close (archive);
+       return NULL;
+      }
 
-  return pos;
+   buf = g_malloc (sb.size);
+   gint bytes = zip_fread (file_zip, buf, sb.size);
+
+   zip_fclose (file_zip);
+   zip_close (archive);
+
+#endif
+
+  return buf;
 }
 
+
+gchar* read_single_file_from_zip (const gchar *archive_name)
+{
+  gchar *buf = NULL;
+
+#ifdef LIBZIP_SUPPORTED
+
+  int errorp;
+  struct zip *archive = zip_open (archive_name, ZIP_CHECKCONS, &errorp);
+  if (! archive)
+     {
+      dbm ("! archive");
+      return NULL;
+     }
+
+  struct zip_file *file_zip = zip_fopen_index (archive, 0, 0);
+  if (! file_zip)
+     {
+      dbm ("! file_zip");
+      zip_close (archive);
+      return NULL;
+     }
+
+   struct zip_stat sb;
+   if (zip_stat_index (archive, 0, 0, &sb) != 0)
+      {
+       dbm ("! zip_stat");
+       zip_fclose (file_zip);
+       zip_close (archive);
+       return NULL;
+      }
+
+   buf = g_malloc (sb.size);
+   gint bytes = zip_fread (file_zip, buf, sb.size);
+
+   zip_fclose (file_zip);
+   zip_close (archive);
+
+#endif
+
+  return buf;
+}
\ No newline at end of file
