@@ -23,6 +23,16 @@ typedef struct
   gchar *d;
 } t_str_quad;
 
+
+typedef struct {
+               gchar *word;
+               gint count;
+              } t_struct_word;
+
+
+typedef gboolean (FilterStrFunc) (const gchar *value);
+
+
 gchar* str_after_sep (const gchar *str, const gchar *sep);
 gchar* str_before_sep (const gchar *str, const gchar *sep);
 gchar* str_face_quotes (const gchar *buf, const gchar *q1, const gchar *q2);
@@ -32,12 +42,11 @@ void str_quad_print (t_str_quad *t);
 void str_quad_free (t_str_quad *quad);
 gchar* str_extract_between (const gchar *source, const gchar *a, const gchar *b);
 
-//now plays: Dom & Kemal - Back for the Future - Moulin Rouge
+//now playing: Dom & Kemal - Back for the Future - Moulin Rouge
 
 gchar* text_load_msdoc (const gchar *filename);
 gchar* get_file_ver_name (const gchar *name);
 size_t tNumber2Roman (guint uiNumber, int bUpperCase, gchar *szOutput);
-//GList* parse_string_to_list (const gchar *s);
 GList* add_to_glist_combined_st (GList *list, const gchar* key, const gchar *value);
 GList* add_to_glist_combined_int (GList *list, const gchar* key, gint value);
 GList* remove_blank_lines (GList *lines);
@@ -78,15 +87,13 @@ gchar* quotes_to_tex (const gchar *buf);
 gchar* tex_paras (const gchar *buf);
 gchar* string_from_glist_sep (GList *list, const gchar *sep);
 gchar* change_file_ext (const gchar *filename, const gchar *new_ext);
-gchar* get_file_ext (const gchar *filename);
 gboolean str_in (const gchar *s, gchar *ext1, ...);
 gchar* read_xml_text (const gchar *filename, const gchar *xml_file, const gchar *para);
-gchar* read_abw_text (const gchar *filename, const gchar *xml_file, const gchar *para);
+gchar* read_abw_text (const gchar *filename, const gchar *para);
 gchar* separated_table_to_latex_tabular (const gchar *st, const gchar *sep);
 gint find_str_index (GList *l, const gchar *s);
 GList* filter_list (GList *list, const gchar *phrase, gboolean incld);
 gchar* kill_str_by_size (const gchar *s, gint val, gboolean killlesser);
-gint glist_get_str_pos (GList *list, const gchar *st);
 gchar* copied_from_browser_to_ftext (const gchar *s, const gchar *pattern);
 gchar* get_charset_from_meta (const gchar *text);
 void gslist_strings_free (GSList *list);
@@ -116,3 +123,17 @@ GList* glist_word_sort_mode (GList *list, gint mode);
 gchar* text_load_rtf (const gchar *filename);
 gchar* enc_guess (const gchar *s);
 GList* add_to_glist_combined_str (GList *list, const gchar *key, const gchar *value);
+GList* glist_copy_with_data (GList *source, gint count);
+GList* glist_copy_with_filter (GList *source, FilterStrFunc func);
+void free_word_data (gpointer data);
+gint sort_node_m1 (t_struct_word *a, t_struct_word *b);
+gint sort_node_m2 (t_struct_word *a, t_struct_word *b);
+gint sort_node_m4 (const gchar *a, const gchar *b);
+GList* glist_word_sort_mode (GList *list, gint mode);
+gchar *apply_repl_table (const gchar *text, const gchar *table_filename);
+void print_ht (GHashTable *ht);
+void save_ht_to_file (GHashTable *hash_table, const gchar *filename);
+gchar* strinfile_charset (const gchar *filename, const gchar *text_to_find, const gchar *charset);
+GList* filter_list_pattern (GList *list, const gchar *pattern, gboolean incld);
+gchar* read_file_from_zip (const gchar *archive_name, const gchar *file_name);
+gchar* read_single_file_from_zip (const gchar *archive_name);
