@@ -109,7 +109,7 @@
 #ifdef FOR_DEBIAN
 
 #define TEA_RUN_CMD "Exec=teaed"
-#define TEA_DOC_DIR "/usr/share/doc/tea/"
+#define TEA_DOC_DIR "/usr/share/doc/tea-data/"
 #define TEA_PIX_MAINLOGO "/usr/share/pixmaps/tea_logo.jpg"
 #define TEA_PIX_MAINICON "/usr/share/pixmaps/tea_icon_v2.png"
 #define TEA_PIX_DIR "/usr/share/pixmaps/"
@@ -134,3 +134,11 @@
 #endif
 
 #define doc_get_buf(x) get_text_buf(x)
+#define get_tag_by_name(doc,name) gtk_text_tag_table_lookup(gtk_text_buffer_get_tag_table(doc->text_buffer),name)
+
+#define DSK_UNKNOWN 0
+#define DSK_KDE 1
+#define DSK_GNOME 2
+#define DSK_XFCE 3
+
+
