@@ -1,15 +1,13 @@
-/***************************************************************************
-                          tea_funx.c  -  description
+/*                          tea_funx.c  -  description
                              -------------------
     begin                : Mon Dec 12 2003
-    copyright            : (C) 2003-2006 by Peter 'Roxton' Semiletov
+    copyright            : (C) 2003-2007 by Peter 'Roxton' Semiletov
     email                : peter.semiletov@gmail.com
  ***************************************************************************/
 
 /*
 Copyright (C) 2002 Paolo Maggi
 Copyright (C) 2000  Kh. Naba Kumar Singh
-Copyright (C) 2005 BMPx development team.
 */
 
 /***************************************************************************
@@ -40,6 +38,10 @@ Copyright (C) 2005 BMPx development team.
 #include <stdio.h>
 #include <string.h>
 
+#ifdef LIBZIP_SUPPORTED
+#include "zip.h"
+#endif
+
 
 #include "tea_defs.h"
 #include "tea_text_document.h" // Needs to be above for t_note_page struct
@@ -54,19 +56,7 @@ Copyright (C) 2005 BMPx development team.
 #include "callbacks.h"
 
 
-/* To avoid implicit declaration warning */
-void make_stats (t_note_page *doc);
-
-
-/* from:
- * time.c
- * This file is part of gedit
- *
- * Copyright (C) 2002 Paolo Maggi
-
-  changed by roxton
-*/
-
+//Copyright (C) 2002 Paolo Maggi
 gchar* get_time (const gchar *format)
 {
    if (! format)
@@ -180,10 +170,16 @@ int get_value (int total, int perc)
   return (int) (total / 100) * perc;
 }
 
-
-int get_percent (int total, int value)
+/*
+double get_value (double total, double perc)
 {
-  return (int) (value / total) * 100;
+  return (total / 100) * perc;
+}
+*/
+
+double get_percent (double total, double value)
+{
+  return (value / total) * 100;
 }
 
 
@@ -263,12 +259,6 @@ void handle_file_enc (const gchar *filename, const gchar *enc)
   open_file_std (filename);
 }
 
-/*
-gboolean is_image2 (const gchar *filename)
-{
-  return is_ext (filename, ".png", ".gif", ".jpg", ".jpeg", ".wbmp", ".bmp", ".mng", NULL);
-}
-*/
 
 gboolean is_tex (const gchar *filename)
 {
@@ -304,7 +294,7 @@ gint get_file_size (const gchar *filename)
 }
 
 
-//check extension without dot
+//check extension without the dot
 gboolean check_ext_wo_dot (const gchar *filename, const gchar *ext)
 {
   if (! filename || ! ext)
@@ -342,7 +332,7 @@ gboolean is_markup (const gchar *f)
 
 gboolean is_c (const gchar *f)
 {
-  return is_ext (f, ".c", ".h", ".cpp", ".cxx", ".cc", ".cp", ".c++", NULL);
+  return is_ext (f, ".c", ".h", ".cpp", ".hh", ".cxx", ".cc", ".cp", ".c++", NULL);
 }
 
 
@@ -423,195 +413,6 @@ gchar* get_8_filename (const gchar *filename)
   return g_filename_to_utf8 (filename, -1, &bytes_read, &bytes_written, NULL);
 }
 
-/*
-void make_stats2 (t_note_page *doc)
-{
-  gboolean selected = TRUE;
-  gchar *text;
-  gint words = 0;
-  gint chars = 0;
-  gint white_chars = 0;
-  gint lines = 0;
-  gint bytes = 0;
-  GtkTextIter start;
-  GtkTextIter end;
-  gunichar ch;
-
-  if (doc_is_sel (doc->text_buffer))
-     gtk_text_buffer_get_selection_bounds (doc->text_buffer, &start, &end);
-  else
-      {
-       gtk_text_buffer_get_bounds (doc->text_buffer, &start, &end);
-       selected = FALSE;
-      }
-
-  lines = gtk_text_buffer_get_line_count (doc->text_buffer);
-  text =  gtk_text_buffer_get_text (doc->text_buffer, &start, &end, FALSE);
-  bytes = strlen (text);
-
-  do
-    {
-     ch = gtk_text_iter_get_char (&start);
-     if (g_unichar_isspace (ch))
-        white_chars++;
-     if (gtk_text_iter_starts_word (&start))
-        words++;
-
-     chars++;
-    }
-  while ( (gtk_text_iter_forward_char (&start)) && ( ! gtk_text_iter_equal (&start, &end)) );
-
-  gchar *s_bytes = g_strdup_printf(_("bytes: %d\n"), bytes);
-  gchar *s_lines = g_strdup_printf(_("lines: %d\n"), lines);
-  gchar *s_words = g_strdup_printf(_("words: %d\n"), words);
-  gchar *s_chars = g_strdup_printf(_("chars: %d\n"), chars);
-  gchar *s_charsnsp = g_strdup_printf(_("chars non-space: %d\n"), chars - white_chars);
-
-  gchar *result;
-
-  if (! selected)
-     result = g_strconcat (_("stats for "),
-                           doc->file_name_utf8,
-                           ":\n",
-                           s_charsnsp,
-                           s_chars,
-                           s_words,
-                           s_lines,
-                           s_bytes,
-                           NULL);
-   else
-       result = g_strconcat (_("stats for the selection"),
-                             ":\n",
-                             s_charsnsp,
-                             s_chars,
-                             s_words,
-                             s_bytes,
-                             NULL);
-
-  log_to_memo (result, NULL, LM_NORMAL);
-
-  g_free (s_bytes);
-  g_free (s_charsnsp);
-  g_free (s_chars);
-  g_free (s_words);
-  g_free (s_lines);
-  g_free (result);
-  g_free (text);
-}
-*/
-
-
-//from Gedit - plugin
-/*
- * docinfo.c
- * This file is part of gedit
- *
- * Copyright (C) 2002 Paolo Maggi
-
-modified by roxton
-*/
-
-void make_stats (t_note_page *doc)
-{
-  if (! doc)
-     return;
-
-  if (! doc->text_buffer)
-     return;
-
-  gboolean selected = TRUE;
-
-  gchar *text;
-  PangoLogAttr *attrs;
-  gint words = 0;
-  gint chars = 0;
-  gint white_chars = 0;
-  gint lines = 0;
-  gint bytes = 0;
-  gint i;
-  gchar *tmp_str;
-
-  selected = doc_has_selection (doc);
-
-  if (! selected)
-      text = doc_get_buf (doc->text_buffer);
-  else
-      text = doc_get_sel (doc);
-
-  if (! text)
-     return;
-
-  if (! g_utf8_validate (text, -1, NULL))
-     return;
-
-  lines = gtk_text_buffer_get_line_count (doc->text_buffer);
-
-  chars = g_utf8_strlen (text, -1);
-  attrs = g_new0 (PangoLogAttr, chars + 1);
-
-  pango_get_log_attrs (text,
-                       -1,
-                       0,
-                       pango_language_from_string ("C"),
-                       attrs,
-                       chars + 1);
-
-  i = 0;
-
-  while (i < chars)
-        {
-         if (attrs [i].is_white)
-            ++white_chars;
-
-         if (attrs [i].is_word_start)
-            ++words;
-
-         ++i;
-        }
-
-  if (chars == 0)
-     lines = 0;
-
-  bytes = strlen (text);
-
-  gchar *s_bytes = g_strdup_printf(_("bytes: %d\n"), bytes);
-  gchar *s_lines = g_strdup_printf(_("lines: %d\n"), lines);
-  gchar *s_words = g_strdup_printf(_("words: %d\n"), words);
-  gchar *s_chars = g_strdup_printf(_("chars: %d\n"), chars);
-  gchar *s_charsnsp = g_strdup_printf(_("chars non-space: %d\n"), chars - white_chars);
-
-  gchar *result;
-
-  if (! selected)
-     result = g_strconcat (_("stats for "),
-                           doc->file_name_utf8,
-                           ":\n",
-                           s_charsnsp,
-                           s_chars,
-                           s_words,
-                           s_lines,
-                           s_bytes,
-                           NULL);
-   else
-       result = g_strconcat (_("stats for selected:\n"),
-                             s_charsnsp,
-                             s_chars,
-                             s_words,
-                             s_bytes,
-                             NULL);
-
-  log_to_memo (result, NULL, LM_NORMAL);
-
-  g_free (s_bytes);
-  g_free (s_charsnsp);
-  g_free (s_chars);
-  g_free (s_words);
-  g_free (s_lines);
-  g_free (result);
-  g_free (attrs);
-  g_free (text);
-}
-
 
 gchar* get_lang_name (void)
 {
@@ -720,71 +521,6 @@ gchar* get_clipboard_text (void)
 }
 
 
-void insert_image (const gchar *a_filename)
-{
-  if (! get_page_text ())
-     return;
-
-  if (! cur_text_doc->b_saved)
-     return;
-
-  if (! a_filename)
-     return;
-
-  if (! g_file_test (a_filename, G_FILE_TEST_EXISTS))
-     return;
-
-  gint width = 0;
-  gint height = 0;
-  gdk_pixbuf_get_file_info (a_filename, &width, &height);
-
-  gchar *filename = create_relative_link (cur_text_doc->file_name_local, a_filename);
-
-  if (! filename)
-      return;
-
-  gint mm = get_markup_mode ();
-  gchar *t;
-
-  gchar *temp = g_malloc (7777);
-
-  if (mm == MM_HTML)
-      g_snprintf (temp,
-                  7777,
-                  "<img src=\"%s\" border=\"0\" alt=\"\" width=\"%d\" height=\"%d\">",
-                  filename, width, height);
-  else
-  if (mm == MM_XHTML)
-     g_snprintf (temp,
-                 7777,
-                 "<img src=\"%s\" border=\"0\" alt=\"\" width=\"%d\" height=\"%d\" />",
-                 filename, width, height);
-  else
-      if (mm == MM_DOCBOOK)
-          g_snprintf (temp,
-                      7777,
-                      "<mediaobject><imageobject>\n<imagedata fileref=\"%s\"/>\n</imageobject></mediaobject>",
-                      filename);
-  else
-  if (mm == MM_WIKI)
-     g_snprintf (temp,
-                 7777,
-                 "[[Image:%s|thumb|alternative text]]",
-                 filename);
-  else
-  if (mm == MM_TEX)
-     g_snprintf (temp,
-                 7777,
-                 "\\includegraphics{%s}",
-                 filename);
-
-  doc_insert_at_cursor (cur_text_doc, temp);
-
-  g_free (temp);
-  g_free (filename);
-}
-
-
 void insert_link (const gchar *a_filename)
 {
   if (! a_filename || ! cur_text_doc)
@@ -1073,7 +809,7 @@ gchar* filename_from_xuri (const gchar *uri)
 }
 
 
-gchar *create_relative_link (const gchar *doc_filename, const gchar *img_filename)
+gchar* create_relative_link (const gchar *doc_filename, const gchar *img_filename)
 {
   if (! doc_filename || ! img_filename)
      return NULL;
@@ -1235,70 +971,6 @@ gboolean has_ext (const gchar *filename)
   return (strstr (++filename, "."));
 }
 
-/*
-from:
-  BMPx - The Dumb Music Player
-  Copyright (C) 2005 BMPx development team.
-*/
-
-
-gboolean dir_foreach (const gchar *path, DirForeachFunc function,
-                      gpointer user_data, GError **error)
-{
-  GDir *dir;
-  const gchar *entry;
-  gchar *entry_fullpath;
-
-  if (! (dir = g_dir_open (path, 0, error)))
-    return FALSE;
-
-  while ((entry = g_dir_read_name (dir)))
-       {
-        entry_fullpath = g_build_filename (path, entry, NULL);
-
-        if ((*function) (entry_fullpath, entry, user_data))
-           {
-            g_free (entry_fullpath);
-            break;
-           }
-
-         g_free (entry_fullpath);
-       }
-
-  g_dir_close (dir);
-  return TRUE;
-}
-
-
-gboolean cbf_listdir (const gchar *path, const gchar *basename, gpointer user_data)
-{
-  GError *error = NULL;
-
-  if (g_file_test (path, G_FILE_TEST_IS_DIR))
-     {
-      dir_foreach (path, cbf_listdir, user_data, &error);
-      return FALSE;
-     }
-
-  if (g_pattern_match_simple (user_data, path))
-      gl_found_files = g_list_prepend (gl_found_files, g_strdup (path));
-
-
-  return FALSE;
-}
-
-
-void read_dir_files_recurse (const gchar *path, const gchar *pattern)
-{
-  if (! path || ! pattern)
-     return;
-
-  glist_strings_free (gl_found_files);
-  gl_found_files = NULL;
-  GError *error = NULL;
-  dir_foreach (path, cbf_listdir, pattern, &error);
-}
-
 
 gchar* get_build_info (void)
 {
@@ -1330,10 +1002,10 @@ gchar* file_combine_path (const gchar *dir, const gchar *filename)
   if (! filename || ! dir)
      return NULL;
 
-  if (g_str_has_suffix  (dir, G_DIR_SEPARATOR_S))
+  if (g_str_has_suffix (dir, G_DIR_SEPARATOR_S))
      return g_strconcat (dir, filename, NULL);
   else
-     return g_strconcat (dir, filename, G_DIR_SEPARATOR_S, NULL);
+     return g_strconcat (dir, G_DIR_SEPARATOR_S, filename, NULL);
 }
 
 
@@ -1349,3 +1021,253 @@ gchar* add_slash_if_need (const gchar *dir)
 }
 
 
+void insert_image (const gchar *a_filename, gint width, gint height, gboolean read_xy_from_file)
+{
+  if (! get_page_text ())
+     return;
+
+  if (! cur_text_doc->b_saved)
+     return;
+
+  if (! a_filename)
+     return;
+
+  if (! g_file_test (a_filename, G_FILE_TEST_EXISTS))
+     return;
+
+  if (read_xy_from_file)
+     gdk_pixbuf_get_file_info (a_filename, &width, &height);
+
+  gchar *filename = create_relative_link (cur_text_doc->file_name_local, a_filename);
+
+  if (! filename)
+      return;
+
+  gint mm = get_markup_mode ();
+  gchar *t;
+
+  gchar *temp = g_malloc (7777);
+
+  if (mm == MM_HTML)
+      g_snprintf (temp,
+                  7777,
+                  "<img src=\"%s\" border=\"0\" alt=\"\" width=\"%d\" height=\"%d\">",
+                  filename, width, height);
+  else
+  if (mm == MM_XHTML)
+     g_snprintf (temp,
+                 7777,
+                 "<img src=\"%s\" border=\"0\" alt=\"\" width=\"%d\" height=\"%d\" />",
+                 filename, width, height);
+  else
+      if (mm == MM_DOCBOOK)
+          g_snprintf (temp,
+                      7777,
+                      "<mediaobject><imageobject>\n<imagedata fileref=\"%s\"/>\n</imageobject></mediaobject>",
+                      filename);
+  else
+  if (mm == MM_WIKI)
+     g_snprintf (temp,
+                 7777,
+                 "[[Image:%s|thumb|alternative text]]",
+                 filename);
+  else
+  if (mm == MM_TEX)
+     g_snprintf (temp,
+                 7777,
+                 "\\includegraphics{%s}",
+                 filename);
+
+  doc_insert_at_cursor (cur_text_doc, temp);
+
+  g_free (temp);
+  g_free (filename);
+}
+
+
+gboolean check_prefix (const gchar *filename, const gchar *ext)
+{
+  if (! filename || ! ext)
+     return FALSE;
+
+  gboolean r = FALSE;
+  gchar *f = g_utf8_strdown (filename, -1);
+  r = g_str_has_prefix (f, ext);
+  g_free (f);
+  return r;
+}
+
+
+gboolean is_prefixed (const gchar *filename, gchar *prefix1, ...)
+{
+  if (! filename)
+     return FALSE;
+
+  va_list args;
+  gchar *t;
+  gboolean r = FALSE;
+
+  va_start (args, prefix1);
+  t = prefix1;
+
+  while (t)
+        {
+         r = check_prefix (filename, t);
+         if (r)
+            break;
+
+         t = va_arg (args, gchar*);
+        }
+
+  va_end (args);
+
+  return r;
+}
+
+
+gchar* path_get_last_element (const gchar *s)
+{
+  if (! s)
+     return NULL;
+
+  gchar *n = g_malloc (strlen (s));
+  if (g_str_has_suffix (s, G_DIR_SEPARATOR_S))
+      n = g_utf8_strncpy (n, s, g_utf8_strlen (s, -1) -1);
+  else
+      {
+       g_free (n);
+       n = g_strdup (s);
+      }
+
+  GList *l = glist_from_string_sep (n, G_DIR_SEPARATOR_S);
+  if (! l)
+      return NULL;
+
+  GList *t = g_list_last (l);
+
+  gchar *x = g_strdup (t->data);
+  glist_strings_free (l);
+  g_free (n);
+  return x;
+}
+
+
+//n.p. Led Zeppelin "Immigrants Song"
+gchar* path_up (const gchar *filename)
+{
+   gint c = 0;
+   gint i = 0;
+
+   while (filename[i++])
+         if (filename[i] == G_DIR_SEPARATOR)
+            c = i;
+
+   if (c == 0)
+      return g_strdup (G_DIR_SEPARATOR_S);
+   else
+       return g_strndup (filename, c);
+}
+
+
+gint get_desktop_name (void)
+{
+  gint res = DSK_UNKNOWN;
+
+  if (g_getenv ("KDE_FULL_SESSION"))
+     res = DSK_KDE;
+  else
+      if (g_getenv ("GNOME_DESKTOP_SESSION_ID"))
+         res = DSK_GNOME;
+
+  if (res != DSK_UNKNOWN)
+     return res;
+
+  gchar *wm = gdk_x11_screen_get_window_manager_name (gdk_screen_get_default ());
+
+  if (wm)
+     {
+       if (strstr (wm, "kwin"))
+          res = DSK_KDE;
+       else
+           if (strstr (wm, "metacity"))
+              res = DSK_GNOME;
+              else
+                  if (strstr (wm, "xfwm4"))
+                     res = DSK_XFCE;
+      }
+}
+
+
+void read_dir_files_rec_inner (const gchar *path)
+{
+  GDir *d = g_dir_open (path, 0, NULL);
+  if (! d)
+      return;
+
+  gchar *t;
+  gchar *x;
+
+  while (t = g_dir_read_name (d))
+         {
+          x = g_strconcat (path, G_DIR_SEPARATOR_S, t, NULL);
+          if (g_file_test (x, G_FILE_TEST_IS_DIR) && ! g_file_test (x, G_FILE_TEST_IS_SYMLINK))
+              read_dir_files_rec_inner (x);
+          else
+              gl_found_files = g_list_prepend (gl_found_files, x);
+         }
+
+  g_dir_close (d);
+}
+
+
+GList* read_dir_files_rec (const gchar *path)
+{
+  glist_strings_free (gl_found_files);
+  gl_found_files = NULL;
+
+  read_dir_files_rec_inner (path);
+  return g_list_reverse (gl_found_files);
+}
+
+
+GList* read_dir_files_rec_pat (const gchar *path, const gchar *pattern)
+{
+  if (! path || ! pattern)
+     return NULL;
+
+  GList *ffiles = read_dir_files_rec (path);
+  GList *files = filter_list_pattern (ffiles, pattern, TRUE);
+  return files;
+}
+
+
+void zip_create_archive (const gchar *filename)
+{
+#ifdef LIBZIP_SUPPORTED
+
+  if (! filename && file_exists (filename))
+     return;
+
+  gchar *archive_name = g_strconcat (filename, ".zip", NULL);
+
+  dbm (archive_name);
+  gint errorp;
+  struct zip *archive = zip_open (archive_name, ZIP_CREATE, &errorp);
+  if (! archive)
+     {
+      dbm ("! archive");
+      return;
+     }
+
+  gchar *fname = g_path_get_basename (filename);
+  struct zip_source *source = zip_source_file (archive, filename, 1, -1);
+
+  if (source)
+     zip_add (archive, fname, source);
+
+  g_free (fname);
+  g_free (archive_name);
+  zip_close (archive);
+
+#endif
+}
