@@ -2,7 +2,7 @@
                           tea_gtk_utils.c  -  description
                              -------------------
     begin                : aug. 2004
-    copyright            : (C) 2004-2006 by Peter 'Roxton' Semiletov
+    copyright            : (C) 2004-2007 by Peter 'Roxton' Semiletov
     email                : peter.semiletov@gmail.com
  ***************************************************************************/
 //  Copyright (C) 1998, 1999 Alex Roberts, Evan Lawrence
@@ -201,8 +201,9 @@ GtkWidget* tea_dir_selector (GtkWidget *container, const gchar *caption, const g
 }
 
 
-//returns combo
-GtkWidget* tea_combo_combo (GtkWidget *container, GList *strings, const gchar *caption, gint value)
+//returns the combo
+GtkWidget* tea_combo_combo (GtkWidget *container, GList *strings, const gchar *caption,
+                            gint value, gpointer on_changed, gpointer user_data)
 {
   GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
   gtk_widget_show (b);
@@ -218,12 +219,17 @@ GtkWidget* tea_combo_combo (GtkWidget *container, GList *strings, const gchar *c
   if (strings)
     fill_combo_with_glist (x, strings);
 
-   gtk_combo_box_set_active (x, value);
+  if (value != -1)
+     gtk_combo_box_set_active (x, value);
+
+  if (on_changed)
+      g_signal_connect (x, "changed", on_changed, user_data);
 
   return x;
 }
 
 
+//returns the text entry
 GtkWidget* tea_combo (GtkWidget *container, GList *strings, const gchar *caption, const gchar *value)
 {
   GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
@@ -253,7 +259,6 @@ GtkWidget* tea_combo (GtkWidget *container, GList *strings, const gchar *caption
 // * Copyright (C) 1998, 1999 Alex Roberts, Evan Lawrence
 // * Copyright (C) 2000, 2002 Chema Celorio, Paolo Maggi
 //mod. by roxton
-
 void widget_set_colors (GtkWidget *w, GdkColor *background, GdkColor *text,
                         GdkColor *selection, GdkColor *sel_text)
 {
@@ -345,6 +350,9 @@ void dlg_info (const gchar *caption, const gchar *prompt, const gchar *val)
 
 GtkWidget* tea_button_at_box (GtkWidget *container, const gchar *caption, gpointer callback, gpointer user_data)
 {
+  if (! container)
+     return NULL;
+
   GtkWidget *b = gtk_button_new_with_label (caption);
   gtk_widget_show (b);
 
@@ -354,10 +362,9 @@ GtkWidget* tea_button_at_box (GtkWidget *container, const gchar *caption, gpoint
                       FALSE,
                       UI_PACKFACTOR);
 
-  g_signal_connect (GTK_OBJECT (b),
-                    "clicked",
-                    G_CALLBACK (callback),
-                    user_data);
+  if (callback)
+      g_signal_connect (GTK_OBJECT (b), "clicked", G_CALLBACK (callback), user_data);
+
   return b;
 }
 
@@ -383,7 +390,6 @@ static void find_menuitem_cb (GtkWidget *widget, gpointer data)
 
   if (GTK_IS_MENU_ITEM (widget))
      find_menuitem (GTK_CONTAINER (gtk_menu_item_get_submenu (widget)), data);
-
 }
 
 
@@ -417,24 +423,6 @@ GtkFileFilter* tea_file_filter_with_patterns (const gchar *name, gchar *pattern1
 }
 
 
-GtkFileFilter* find_ffilter_by_name (GtkFileChooser *chooser, const gchar *name)
-{
-  GtkFileFilter *f = NULL;
-  GSList *l = gtk_file_chooser_list_filters (chooser);
-
-  GList *t = l;
-  while (t && t->data)
-       {
-        if (g_utf8_collate (gtk_file_filter_get_name (l->data), name) == 0)
-           f = l->data;
-        t = g_slist_next (t);
-       }
-
-  g_slist_free (l);
-  return f;
-}
-
-
 GtkWidget* daisho_add_del (gpointer *f1, gpointer f2)
 {
   GtkWidget *vb = gtk_vbox_new (FALSE, UI_PACKFACTOR);
@@ -461,9 +449,7 @@ GtkWidget* daisho_add_del (gpointer *f1, gpointer f2)
 
 GtkWidget* new_menu_item_with_udata (const gchar *label, GtkWidget *parent, gpointer f, gpointer data)
 {
-  GtkWidget *item;
-
-  item = gtk_menu_item_new_with_label (label);
+  GtkWidget *item = gtk_menu_item_new_with_label (label);
   gtk_widget_show (item);
   gtk_container_add (GTK_CONTAINER (parent), item);
   gtk_widget_set_name (item, label);
@@ -477,10 +463,7 @@ GtkWidget* new_menu_item_with_udata (const gchar *label, GtkWidget *parent, gpoi
 
 GtkWidget* new_menu_item (const gchar *label, GtkWidget *parent, gpointer f)
 {
-  GtkWidget *item;
-
-  item = gtk_menu_item_new_with_label (label);
-
+  GtkWidget *item = gtk_menu_item_new_with_label (label);
   gtk_widget_show (item);
   gtk_container_add (GTK_CONTAINER (parent), item);
   gtk_widget_set_name (item, label);
@@ -492,35 +475,6 @@ GtkWidget* new_menu_item (const gchar *label, GtkWidget *parent, gpointer f)
 }
 
 
-//new_action_item returns menu item
-GtkWidget* new_action_item (GtkActionGroup *ag,
-                            GtkWidget *parent,
-                            const gchar *name,
-                            const gchar *label,
-                            const gchar *tooltip,
-                            const gchar *stock_id,
-                            gpointer f)
-{
-  GtkWidget *item;
-  GtkAction *action = gtk_action_new (name,
-                                      label,
-                                      tooltip,
-                                      stock_id);
-
-  item = gtk_action_create_menu_item (action);
-  gtk_widget_show (item);
-  gtk_container_add (GTK_CONTAINER (parent), item);
-  gtk_widget_set_name (item, label);
-
-  if (f)
-     g_signal_connect ((gpointer) item, "activate", G_CALLBACK (f), NULL);
-
-  gtk_action_group_add_action (ag, action);
-
-  return item;
-}
-
-
 GtkWidget* new_menu_sep (GtkWidget *parent)
 {
   GtkWidget *mni_sep = gtk_menu_item_new ();
@@ -580,16 +534,17 @@ gboolean dlg_key_handler (GtkWidget *widget, GdkEventKey *event, GtkWidget *w)
  *
  * Return value: void
  **/
-void window_destroy(GtkWidget * windowname) {
-	g_signal_handlers_destroy(G_OBJECT(windowname));
-	gtk_grab_remove(windowname);
-	gtk_widget_destroy(windowname);
-       }
+void window_destroy (GtkWidget *windowname)
+     {
+      g_signal_handlers_destroy (G_OBJECT(windowname));
+      gtk_grab_remove (windowname);
+      gtk_widget_destroy (windowname);
+     }
 ////////
 
 void set_cursor_blink (gboolean value)
 {
-   g_object_set (G_OBJECT ( gtk_settings_get_default ()),
+  g_object_set (G_OBJECT ( gtk_settings_get_default ()),
                "gtk-cursor-blink",
                 value,
                 NULL);
@@ -677,8 +632,8 @@ gchar* dlg_question_char (const gchar *prompt, const gchar *defval)
   GtkWidget *ent = tea_text_entry (GTK_DIALOG (dialog)->vbox, prompt, defval);
 
   if (gtk_dialog_run (dialog) == GTK_RESPONSE_ACCEPT)
-  if (gtk_entry_get_text (ent))
-      result = g_strdup (gtk_entry_get_text (ent));
+     if (gtk_entry_get_text (ent))
+         result = g_strdup (gtk_entry_get_text (ent));
 
   gtk_widget_destroy (dialog);
 
@@ -767,11 +722,6 @@ GtkWidget* tea_color_selector_simple (GtkWidget *container, const gchar *caption
   return bt;
 }
 
-gchar* get_gtk_ver (void)
-{
-  return g_strdup_printf ("%d.%d.%d", gtk_major_version, gtk_minor_version, gtk_micro_version);
-}
-
 
 gchar* get_text_buf (gpointer text_buffer)
 {
@@ -779,9 +729,67 @@ gchar* get_text_buf (gpointer text_buffer)
      return NULL;
 
   GtkTextIter itstart, itend;
-
-  gtk_text_buffer_get_iter_at_offset (text_buffer, &itstart, 0);
-  gtk_text_buffer_get_iter_at_offset(text_buffer, &itend, gtk_text_buffer_get_char_count (text_buffer));
+  gtk_text_buffer_get_bounds (text_buffer, &itstart, &itend);
 
   return gtk_text_buffer_get_text (text_buffer, &itstart, &itend, FALSE);
 }
+
+
+static gboolean on_shortcut_entry_keypress (GtkWidget *widget,
+                                            GdkEventKey *event,
+                                            gpointer data)
+{
+  GString *s = g_string_sized_new (64);
+
+  if (event->state & GDK_CONTROL_MASK)
+     s = g_string_append (s, "Ctrl ");
+
+  if (event->state & GDK_SHIFT_MASK)
+     s = g_string_append (s, "Shift ");
+
+  if (event->state & GDK_MOD1_MASK)
+     s = g_string_append (s, "Alt ");
+
+  gchar *key = gdk_keyval_name (event->keyval);
+  s = g_string_append (s, key);
+  gchar *val = g_string_free (s, FALSE);
+
+  gtk_entry_set_text (data, val);
+  g_free (val);
+
+  return TRUE;
+}
+
+
+GtkWidget* tea_shortcut_entry (void)
+{
+  GtkWidget *x = gtk_entry_new ();
+  gtk_widget_show (x);
+
+  g_signal_connect (x, "key-press-event",
+                    G_CALLBACK (on_shortcut_entry_keypress), x);
+
+  return x;
+}
+
+
+GtkWidget* tea_text_entry_hotkey (GtkWidget *container, const gchar *caption, const gchar *value)
+{
+  GtkWidget *b = gtk_hbox_new (FALSE, UI_PACKFACTOR);
+  gtk_widget_show (b);
+  gtk_box_pack_start (container, b, FALSE, TRUE, UI_PACKFACTOR);
+
+  GtkWidget *x = gtk_label_new (caption);
+  gtk_label_set_use_markup (x, TRUE);
+
+  gtk_widget_show (x);
+  gtk_box_pack_start (b, x, FALSE, FALSE, UI_PACKFACTOR);
+  x = tea_shortcut_entry ();
+  gtk_widget_show (x);
+  gtk_box_pack_start (b, x, TRUE, TRUE, UI_PACKFACTOR);
+
+  if (value)
+     gtk_entry_set_text (x, value);
+
+  return x;
+}
