@@ -2,7 +2,7 @@
                           tea_kwas.c  -  description
                              -------------------
     begin                : 11.06.2004 - it was a really good day
-    copyright            : (C) 2004-2006 by Peter 'Roxton' Semiletov
+    copyright            : (C) 2004-2007 by Peter 'Roxton' Semiletov
     email                : peter.semiletov@gmail.com
  ***************************************************************************/
 /*
@@ -85,9 +85,9 @@ typedef struct
 
 //from Beaver's an Early AdVanced EditoR fileop.c
 /* Return the rwx permissions of a file in a string */
-static gchar* get_file_mode (struct stat *Stats)
+gchar* get_file_mode (struct stat *Stats)
 {
-  static gchar Mode[10];
+  gchar Mode[10];
 
   g_snprintf (Mode, 10, "%c%c%c%c%c%c%c%c%c",
 	      (Stats->st_mode & S_IRUSR) ? 'r' : '-',
@@ -225,7 +225,7 @@ static void on_mni_kwas_insert_image (GtkMenuItem *menuitem,
   if (! f)
      return;
 
-  insert_image (f);
+  insert_image (f, 0, 0, TRUE);
   g_free (f);
 }
 
@@ -265,21 +265,6 @@ static void on_mni_kwas_copy_filename (GtkMenuItem *menuitem,
 }
 
 
-/*
-static void on_mni_kwas_add_bookmark (GtkMenuItem *menuitem,
-                                      gpointer user_data)
-{
-  t_kwas_win *w = user_data;
-  gchar *dir = g_strconcat ("file:///", w->focused_panel->path, NULL);
-  gl_tea_kwas_bookmarks = g_list_prepend (gl_tea_kwas_bookmarks, dir);
-  glist_save_to_file (gl_tea_kwas_bookmarks, confile.tea_kwas_bookmarks);
-  glist_strings_free (gl_tea_kwas_bookmarks);
-  gl_tea_kwas_bookmarks = load_file_to_glist (confile.tea_kwas_bookmarks);
-  kwas_bookmarks_menu_update (w);
-}
-*/
-
-
 //I am not sure - maybe better to move it into destroy_event?
 static gboolean on_kwas_window_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
 {
@@ -411,23 +396,6 @@ static struct stat* get_stat (const gchar *filename)
 }
 
 
-//n.p. Led Zeppelin "Immigrants Song"
-static gchar* path_up (const gchar *filename)
-{
-   gint c = 0;
-   gint i = 0;
-
-   while (filename[i++])
-         if (filename[i] == G_DIR_SEPARATOR)
-            c = i;
-
-   if (c == 0)
-      return g_strdup (G_DIR_SEPARATOR_S);
-   else
-       return g_strndup (filename, c);
-}
-
-
 static void statusbar_message (t_kwas_win *w, const gchar *s)
 {
   if (! w->status_bar)
@@ -458,6 +426,9 @@ static void filelist_add_new_item (t_kwas_panel *p, const gchar *s)
   GtkTreeModel *model;
   GtkTreeIter newrow;
   t_flist_item *t = g_hash_table_lookup (p->hash_filelist, s);
+  if (! t)
+     return;
+
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (p->tree_view));
 
   gtk_list_store_append (GTK_LIST_STORE (model), &newrow);
@@ -477,7 +448,6 @@ static t_flist_item* fli_new (const gchar *path, const gchar *filename)
 
 #ifndef CHAI_MODE
 
-
 #if defined(DARWIN)
   if (sizeof (off_t) == 8)
       tfl->file_size = g_strdup_printf ("%-10llu", tfl->fstat->st_size);
@@ -493,7 +463,6 @@ static t_flist_item* fli_new (const gchar *path, const gchar *filename)
      tfl->file_size = g_strdup_printf (_("%-10d Kb"), (tfl->fstat->st_size / 1024));
 #endif
 
-
 #else
 
   GnomeVFSFileInfo *info;
@@ -569,6 +538,8 @@ void fill_filelist (t_kwas_panel *p, const gchar *path)
          z = g_strconcat (path, G_DIR_SEPARATOR_S, t, NULL); //full localized path
 
          y = get_8_filename (t); //filename in UTF-8
+         if (! y)
+            continue;
 
          if (g_file_test (z, G_FILE_TEST_IS_DIR))
             {
@@ -630,6 +601,7 @@ static void dir_up (t_kwas_panel *p)
   g_free (p->path);
   p->path  = f;
   fill_filelist (p, p->path);
+  p->pos = 1;
 }
 
 
@@ -781,7 +753,6 @@ static gint button_press_event (GtkWidget *widget, GdkEventButton *event, gpoint
              g_free (name);
              return TRUE;
             }
-
         }
   return FALSE;
 }
@@ -841,18 +812,6 @@ t_kwas_panel* create_filenav_panel (t_kwas_win *w)
   p->tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list_store));
   renderer = gtk_cell_renderer_text_new ();
 
-/*
-  g_object_set(renderer,
-               "cell-background", "navy",
-               "cell-background-set", TRUE,
-               NULL);
-
-  g_object_set(renderer,
-               "foreground", "white",
-               "foreground-set", TRUE,
-               NULL);
-*/
-
   g_object_set (renderer, "height", widget_get_y (GTK_WIDGET (p->tree_view), "ZORROTEST") + 5, NULL);
 
   col = gtk_tree_view_column_new ();
@@ -884,11 +843,6 @@ t_kwas_panel* create_filenav_panel (t_kwas_win *w)
   sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (p->tree_view));
 
   gtk_tree_selection_set_mode (sel, GTK_SELECTION_SINGLE);
-  //gtk_tree_selection_set_mode (sel, GTK_SELECTION_MULTIPLE);
-
-//  g_object_set(renderer, "foreground", "white", "foreground-set", TRUE, NULL);
-//  g_object_set(renderer, "background", "navy", "background-set", TRUE, NULL);
-
   g_signal_connect ((gpointer) p->tree_view, "key_press_event",
                     G_CALLBACK (on_treeview_key_press_event),
                     p);
@@ -1006,10 +960,6 @@ t_kwas_win* create_kwas_window (void)
 
   mni_temp = new_menu_item_with_udata (_("Run with..."), kwas_mn_functions, on_mni_kwas_run_with, w);
   mni_temp = new_menu_item_with_udata (_("Copy current file name"), kwas_mn_functions, on_mni_kwas_copy_filename, w);
-//  mni_temp = new_menu_item_with_udata (_("Bookmark the current dir"), kwas_mn_functions, on_mni_kwas_add_bookmark, w);
-//  mni_temp = new_menu_item_with_udata (_("Generate the file list for Cinelerra"), kwas_mn_functions, on_mni_kwas_cin_filelist, w);
-
-
 
   mni_temp = new_menu_item (_("Navigate"), mn_kwas_menu, NULL);
   mn_kwas_nav_menu = new_menu_submenu (mni_temp);
@@ -1017,7 +967,6 @@ t_kwas_win* create_kwas_window (void)
 
   mni_temp = new_menu_item_with_udata (_("Go to the home directory"), mn_kwas_nav_menu, on_mni_kwas_goto_homedir, w);
 
-
   w->mni_bookmarks = new_menu_item (_("Bookmarks"), mn_kwas_menu, NULL);
   w->mni_bookmarks_menu = new_menu_submenu (w->mni_bookmarks);
 
@@ -1091,7 +1040,7 @@ t_kwas_win* create_kwas_window (void)
   gtk_widget_show (w->cm_entry);
   gtk_box_pack_start (GTK_BOX (hbox1), w->cm_entry, TRUE, TRUE, UI_PACKFACTOR);
 
-  bt_go = gtk_button_new_from_stock (GTK_STOCK_EXECUTE);
+  bt_go = gtk_button_new_with_label (_("Go to"));
   gtk_widget_show (bt_go);
   gtk_box_pack_start (GTK_BOX (hbox1), bt_go, FALSE, FALSE, UI_PACKFACTOR);
   g_signal_connect ((gpointer) bt_go, "clicked",
@@ -1153,4 +1102,3 @@ t_kwas_win* create_kwas_window (void)
   w->msg_counter = 0;
   return w;
 }
-
