@@ -3,9 +3,13 @@
 #endif
 
 #ifdef HAVE_LIBASPELL
-
-void setup_spellchecker (void);
 GList* get_speller_modules_list (const gchar *default_lang);
 GList* get_suggestions_list (const gchar *lang, const gchar *s);
-
 #endif
+
+#ifdef ENCHANT_SUPPORTED
+gboolean enchant_check (const gchar *word);
+void enchant_init (void);
+void enchant_done (void);
+GList* enchant_get_suggestions_list (const gchar *s);
+#endif
\ No newline at end of file
