@@ -2,7 +2,7 @@
                           tea_text_document.h  -  description
                              -------------------
     begin                : Mon Dec 1 2003
-    copyright            : (C) 2003-2006 by Peter 'Roxton' Semiletov
+    copyright            : (C) 2003-2007 by Peter 'Roxton' Semiletov
     email                : peter.semiletov@gmail.com
  ***************************************************************************/
 
@@ -25,25 +25,9 @@
 #include <gtksourceview/gtksourceview.h>
 
 GtkSourceLanguagesManager *source_languages_manager;
-GtkSourceTagTable *main_tags_table;
-
-#else
-
-GtkTextTagTable *main_tags_table;
 
 #endif
 
-GtkTextTag *tag_paint;
-GtkTextTag *tag_current_line;
-GtkTextTag *tag_spell_err;
-GtkTextTag *tag_comment;
-GtkTextTag *tag_identifier;
-GtkTextTag *tag_digit;
-GtkTextTag *tag_string;
-GtkTextTag *tag_html_tag;
-GtkTextTag *tag_preprocessor;
-GtkTextTag *tag_type;
-
 GHashTable *c_t_keywords;
 GHashTable *c_t_types;
 GHashTable *pas_t_keywords;
@@ -177,7 +161,7 @@ gboolean doc_search_f_ncase (t_note_page *doc, const gchar *text);
 gchar* upcase_each_first_letter_ofw (t_note_page *doc);
 gchar* doc_get_word_at_left (t_note_page *doc, GtkTextIter *itstart, GtkTextIter *itend);
 void doc_move_cursor_backw (t_note_page *doc, gint i);
-void doc_move_cursor_forw (t_note_page *doc, gint i);
+//void doc_move_cursor_forw (t_note_page *doc, gint i);
 void doc_move_cursor_backw_middle_tags (t_note_page *doc);
 
 void indent_real(GtkWidget *text_view);
@@ -204,10 +188,9 @@ gchar* get_c_url2 (t_note_page *doc);
 gchar* doc_get_cur_mispelled (t_note_page *doc, GtkTextIter *itstart, GtkTextIter *itend);
 void hide_error_marks (t_note_page *doc);
 
-
+#ifndef CHAI_MODE
 GtkTextTag *tag_paint;
 GtkTextTag *tag_current_line;
-
 GtkTextTag *tag_spell_err;
 GtkTextTag *tag_comment;
 GtkTextTag *tag_identifier;
@@ -226,7 +209,7 @@ GHashTable *py_t_keywords;
 GHashTable *php_t_keywords;
 GHashTable *bash_t_keywords;
 GHashTable *po_t_keywords;
-
+#endif
 
 #ifdef CHAI_MODE
 
@@ -247,10 +230,8 @@ gboolean doc_move_to_next_word_sel_if_regexp (t_note_page *doc, const gchar *tex
 gboolean doc_search_f_silent (t_note_page *doc, const gchar *text);
 void doc_update_statusbar (t_note_page *doc);
 void do_hl_spell_check (t_note_page *doc, const gchar *lang);
-gboolean doc_save_buffer_to_file_l (gpointer text_buffer, const gchar *filename);
 gboolean doc_save_buffer_to_file_iconv (gpointer text_buffer, const gchar *filename, const gchar *enc);
 void doc_set_new_text (t_note_page *doc, GtkTextIter *itstart, GtkTextIter *itend, const gchar *text);
-t_note_page* doc_is_already_opened (const gchar *filename);
 gboolean doc_has_selection (t_note_page *doc);
 void update_cursor_position (GtkTextBuffer *text_buffer, gpointer user_data);
 void
@@ -270,4 +251,15 @@ void doc_toggle_images_visibility (t_note_page *p);
 void remove_tags (t_note_page *doc);
 gboolean doc_search_f_next (t_note_page *doc);
 gint find_index_by_page (t_note_page *page);
+void make_stats (t_note_page *doc);
+void run_unitaz (t_note_page *page, gint sort_type, gboolean case_insensetive);
+void run_extract_words (t_note_page *page);
+void walk_by_words (gpointer key,
+                    gpointer value,
+                    gpointer user_data);
+void add_recent_internal (t_note_page *doc);
+void assign_tags (t_note_page *doc);
+
+gchar* readability_ari_en (t_note_page *page); //The Automated Readability Index (ARI)
+void make_readability (t_note_page *page);
 
