@@ -101,6 +101,8 @@ gchar* tv_get_selected_single (GtkTreeView *tree_view)
      return NULL;
 
   GtkTreeSelection *sl = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
+  if (! sl)
+     return NULL;
 
   if (gtk_tree_selection_get_mode (sl) != GTK_SELECTION_SINGLE)
      return NULL;
@@ -190,39 +192,7 @@ GList* tv_get_list_data (GtkTreeView *tree_view)
 }
 
 
-/////////////////////////////
-
-
-static void tv_simple_cb_img (GtkTreeModel *model,
-                          GtkTreePath *path,
-                          GtkTreeIter *iter,
-                          gpointer data)
-{
-  gchar *name;
-  gtk_tree_model_get (model, iter, COL_TEXT_CELL, &name, -1);
-  tv_temp_list = g_list_prepend (tv_temp_list_img, name);
-}
-
-
-GList* tv_get_selected_multiply_img (GtkTreeView *tree_view)
-{
-  if (tv_temp_list_img)
-     g_list_free (tv_temp_list_img);
-
-  tv_temp_list_img = NULL;
-
-  GtkTreeSelection *sl = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
-
-  if (gtk_tree_selection_get_mode (sl) != GTK_SELECTION_MULTIPLE)
-     return NULL;
-
-  gtk_tree_selection_selected_foreach (sl, tv_simple_cb_img, NULL);
-
-  return g_list_reverse (tv_temp_list_img);
-}
-
-
-void tv_add_new_item_img (GtkTreeView *tree_view, gchar *filename, GdkPixbuf *pixbuf)
+void tv_add_new_item_img (GtkTreeView *tree_view, const gchar *filename, GdkPixbuf *pixbuf)
 {
   if (! tree_view || ! filename || ! pixbuf)
      return;
@@ -260,27 +230,6 @@ void tv_fill_with_glist_img (GtkTreeView *tree_view, GList *l)
 }
 
 
-gchar* tv_get_selected_single_img (GtkTreeView *tree_view)
-{
-  if (! tree_view)
-     return NULL;
-
-  GtkTreeSelection *sl = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
-
-  if (gtk_tree_selection_get_mode (sl) != GTK_SELECTION_SINGLE)
-     return NULL;
-
-  GtkTreeModel *model;
-  GtkTreeIter iter;
-  gchar *name = NULL;
-
-  if (gtk_tree_selection_get_selected (sl, &model, &iter))
-      gtk_tree_model_get (model, &iter, COL_TEXT_CELL, &name, -1);
-
-  return name;
-}
-
-
 GtkTreeView* tv_create_img (GtkSelectionMode mode)
 {
   GtkTreeView *tree_view;
@@ -301,9 +250,7 @@ GtkTreeView* tv_create_img (GtkSelectionMode mode)
   gtk_tree_view_column_add_attribute (col, renderer, "pixbuf", COL_IMG_CELL);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), col);
 
-
-  renderer = gtk_cell_renderer_text_new();
-
+  renderer = gtk_cell_renderer_text_new ();
   col = gtk_tree_view_column_new ();
 
   gtk_tree_view_column_pack_start (col, renderer, TRUE);
@@ -318,7 +265,7 @@ GtkTreeView* tv_create_img (GtkSelectionMode mode)
 }
 
 
-GtkTreeView* tv_create_framed_img (GtkContainer *c, gchar *caption, GtkSelectionMode mode)
+GtkTreeView* tv_create_framed_img (GtkContainer *c, const gchar *caption, GtkSelectionMode mode)
 {
   GtkWidget *wnd = gtk_scrolled_window_new (NULL, NULL);
   gtk_widget_show (wnd);
@@ -354,28 +301,36 @@ GtkTreeView* tv_create_tree (GtkContainer *c, GtkSelectionMode mode)
   return tv;
 }
 
-static gboolean cb_listitems_img   (GtkTreeModel *model,
-                                             GtkTreePath *path,
-                                             GtkTreeIter *iter,
-                                             gpointer data)
-{
-  gchar *name;
-  gtk_tree_model_get (model, iter, COL_TEXT, &name, -1);
-  tv_temp_list = g_list_prepend (tv_temp_list, name);
 
-  return FALSE;
+GtkTreeView* tv_create_non_framed (GtkContainer *c, const gchar *caption, GtkSelectionMode mode)
+{
+  GtkWidget *wnd = gtk_scrolled_window_new (NULL, NULL);
+  gtk_widget_show (wnd);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (wnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+
+  GtkTreeView *tv = tv_create (mode);
+  gtk_widget_show (tv);
+  gtk_scrolled_window_add_with_viewport (wnd, tv);
+  gtk_container_add (c, wnd);
+
+  return tv;
 }
 
 
-GList* tv_get_list_data_img (GtkTreeView *tree_view)
+void tv_clear (GtkTreeView *tv)
 {
-  if (tv_temp_list_img)
-     g_list_free (tv_temp_list_img);
+  if (! tv)
+     return;
 
-  tv_temp_list = NULL;
+  GtkTreeModel *model = gtk_tree_view_get_model (tv);
+  if (! model)
+     return;
 
-  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
-  gtk_tree_model_foreach (GTK_TREE_MODEL (model), cb_listitems_img, NULL);
+  GtkTreeIter iter;
 
-  return g_list_reverse (tv_temp_list_img);
+  if (! gtk_tree_model_get_iter_first (model, &iter))
+     return;
+
+  while (gtk_list_store_remove (model, &iter))
+        ;
 }
